#pragma once
#include <atomic>
#include <cstdint>

#include "misc/tick.h"
#include "misc/singleton/singleton.h"

class clip;
class clip_instance;

class CORE_API midi_sequencer : public singleton_t<midi_sequencer> {
public:
    static constexpr int32_t TPQ = 960;
    void test();

    void init(singleton_initliazer& initliazer) override;
    void process(double sample_rate, uint32_t delta_frames);
    auto toggle_play() -> bool ;

    template<class T>
    auto create_clip() -> T* {
        auto clip = new T();
        clips_.push_back(clip);
        return clip;
    }
    auto create_clip_instance(clip* clip) -> clip_instance*;

    void delete_clip(clip* clip);

    auto get_name() -> const char* override { return "midi_sequencer"; }
    void set_bpm(double in_bpm);
    [[nodiscard]] auto get_bpm() const -> double {
        return bpm_;
    }

    [[nodiscard]] auto get_sample_pos() const -> double {
        return sample_pos_;
    }
protected:
    void on_sample_rate_changed(uint32_t sample_rate);
    void update_time(double sample_rate, uint32_t delta_frames);
private:
    void update_vst2_time_info();

    std::atomic<bool> playing_ = false;
    double bpm_ = 0;
    double sample_pos_ = 0;
    int32_t tpq_pos_ = 0;
    uint64_t nano_seconds_ = 0;
    std::vector<clip*> clips_;
    std::vector<clip_instance*> clip_instances_;
};

DEFINE_SINGLETON_INSTANCE(midi_sequencer)
