#include "latency_compensator.h"

#include <cstring>
#include <ranges>
#include <vector>
#include <memory>

#include "audio/device/audio_device_manager.h"
#include "extern.h"
#include "thread_message/thread_message_hubs.h"

latency_compensator::latency_compensator() {
}

latency_compensator::~latency_compensator() {
}

void latency_compensator::init() {
    const uint32_t channel_count = g_audio_device_manager.get_output_channel_count();
    internal_buffer.resize(channel_count);

    uint32_t block_size = g_audio_device_manager.get_buffer_size();
    resize(block_size);
}

void latency_compensator::set_latency(uint32_t in_latency) {
    const uint32_t block_size = g_audio_device_manager.get_buffer_size();
    if (in_latency > 0)
        in_latency += block_size;
    const int32_t need_block = std::ceil((float)in_latency / block_size);
    const int32_t block_num = need_block + 3;
    resize(block_num * block_size);
    push_zeros(in_latency);

    latency_ = in_latency;
}

void latency_compensator::push(audio_buffer& in_buffer) {
    auto headers = in_buffer.get_headers();
    for (uint32_t i = 0; i < internal_buffer.size(); ++i) {
        internal_buffer[i].push(headers[i], in_buffer.get_num_samples());
    }
}

void latency_compensator::pop(uint64_t delta_sample, circular_buffer_vector_type& out_buffer) {
    std::vector<sample_t> temp_buffer(delta_sample);

    for (auto buffer : std::views::zip(internal_buffer, out_buffer)) {
        auto& [internal, target_buffer] = buffer;
        internal.pop(temp_buffer.data(), delta_sample);
        target_buffer.push(temp_buffer);

        memset(temp_buffer.data(), 0, delta_sample * sizeof(sample_t));
    }
}

void latency_compensator::pop(audio_buffer& out_buffer) {
    for (auto buffer : std::views::zip(internal_buffer, out_buffer.get_headers_vector())) {
        auto& [internal, target_buffer] = buffer;
        internal.pop(target_buffer, out_buffer.get_num_samples());
    }
}
