#pragma once
#include <algorithm>
#include <cstdint>

#include "audio/misc/audio_buffer.h"
#include "audio/misc/circular_audio_buffer.h"
#include "misc/delegates.h"

inline multicast_delegate<> on_latency_offset_changed;

class latency_compensator {
public:
    latency_compensator();
    ~latency_compensator();

    void init();

    void push(audio_buffer& in_buffer);
    void pop(uint64_t delta_sample, circular_buffer_vector_type& out_buffer);
    void pop(audio_buffer& out_buffer);

    void set_latency(uint32_t in_latency);
    [[nodiscard]] uint32_t get_latency() const { return latency_; }

    [[nodiscard]] circular_buffer_vector_type& get_internal_buffer() { return internal_buffer; }

protected:
    void resize(uint32_t block_size) {
        for (auto& channel : internal_buffer) {
            channel.set_capacity(block_size);
        }
    }
    // 在缓冲区前面补0, 相当于整个缓冲区向后移动
    void push_zeros(uint32_t num_samples) {
        for (auto& channel : internal_buffer) {
            channel.push_zeros(num_samples);
        }
    }
private:
    // 预计完成的sample_pos
    uint64_t finish_sample_pos_ = 0;
    uint32_t latency_ = 0; // 实际延迟
    circular_buffer_vector_type internal_buffer;
};
